"""
Parsed environment variables, reexported under the same names

Use this module instead of parsing manually:
- to avoid parse logic duplication,
- to make it much easier to find all references,
- to have IDE autofill support :D
"""

import os
